                                                                         /*
Automatic Coffee Fill Solenoid Valve Control Timer
v1.1
by: Don Froula
5/17/2015 
*/


#include <Keypad.h> // This library makes it easy to use a key matrix
#include <EEPROM.h> //This library allows reading and writing to the EEPROM
#include <SimpleTimer.h> //This library allows simple timer functions without interfering with keypad.h timer use

const byte ROWS = 1; // Three rows of keys
const byte COLS = 3;  // One column - Three keys

//Number the buttons of the keypads,
//makes the code easier to understand down the line.
char hexaKeys[ROWS][COLS] = {
  {'1','2','3',}
};

  //R1-C1 keypad pinout
  byte rowPins[ROWS] = {9}; //connect to the row pinouts of the keypad (common)
  byte colPins[COLS] = {8,7,6}; //connect to the column pinouts of the keypad (8=start, 7=up, 6 = down)
  
  //Status LED and valve control Pin definition
  int ledPin = 5; //Pin of status LED
  //int ledPin = 13; //Pin of status LED for test
  int valvePin = 10; //Pin of solenoid valve control


//Initialize an instance of class NewKeypad
Keypad customKeypad = Keypad(makeKeymap(hexaKeys), rowPins, colPins, ROWS, COLS); 

const int timercountEE = 0; //EEPROM memory location for storing seconds timer

int idletimer; //Specific timer ID for idle timer
int filltimer; //Specific timer ID for fill timer
int blinktimer; //Specific timer ID for blink timer
long idle_timer_interval = 600; //Idle timer interval in seconds, 10 minutes
//long idle_timer_interval = 15; //Idle timer interval in seconds, 15 seconds for test

//Instantiate timer - No interrupts or hardware timers used with SimpleTimer! One instance used for multiple timers.
SimpleTimer coffeeTimer;

long timer_delay; //Storage for current fill timer delay

int idlemode = 0; //Idle timer run status flag to indicate wait lockout of next fill to allow water reheating and brew completion
int fillmode = 0; //Fill timer run status flag to indicate active activation of solenoid valve
int blinkmode = 0; //Blink LED mode status flag to indicate in-progress LED fill timer readout

void setup(){
  Serial.begin(9600); // Initialize serial port for debugging

  customKeypad.setHoldTime(2000); // hold any button for 2.0 seconds to change state to HOLD
  
  //Set a listener event subroutine for this keypad
  customKeypad.addEventListener(keypadEvent);
  
  //Initialize and turn off status LED
  pinMode(ledPin, OUTPUT);
  digitalWrite(ledPin, LOW);
  
  //Initialize solenoid valve control to off
  pinMode(valvePin, OUTPUT);
  digitalWrite(valvePin, LOW);
  
  //Read stored fill timer value and initialize on first run or on error
  EEPROM.get(timercountEE, timer_delay); //Retrieve current stored fill_timer value
  if (timer_delay < 2 || timer_delay > 300){ //Check if in range
    timer_delay = 60; //If not in range, set to 60 seconds
    EEPROM.put(timercountEE, timer_delay);} //Write initialized value to EEPROM
    
blinktimer = coffeeTimer.setInterval(500, callback_Blink); //Set repeating blink timer call every .5 second for LED blink in idlemode lockout

delay(500); //Pause after power-up or reset before blinking delay setting

blink_filltime(); //Initiate LED blink of current 3-digit fill_timer setting
 
}
  
  
//Main loop  
void loop(){
coffeeTimer.run(); //Poll coffee timers
char customKey = customKeypad.getKey();  //Poll keypad
}
//End of loop


//Event listener for keypad events.
void keypadEvent(KeypadEvent key){
    switch (customKeypad.getState()){
    case PRESSED:
           switch (key) {
            case '1': //Noop - hold button to start brew cycle
            break;
            
            case '2': //Increase time by two seconds
            if (fillmode == 0  && idlemode == 0 && blinkmode == 0) { //Allow upward 2 second fill time adjustment only if not in fill, delay, or blink out modes
              timer_delay = timer_delay + 2; //Increment time by 2 seconds for every button press
              if (timer_delay > 300) {timer_delay = 300;} //Range check - 5 minute maximum
              EEPROM.put(timercountEE, timer_delay); //Write out new value to non-volatile EEPROM after every button press
              shortBlink(); //Short blink of LED for each button press as confirmation
              }
            break;
            
            case '3': //Decrease time by two seconds
            if (fillmode == 0  && idlemode == 0 && blinkmode == 0) { //Allow downward 2 second fill time adjustment only if not in fill, delay, or blink out modes
              timer_delay = timer_delay - 2; //Decrement time by 2 seconds for every button press
              if (timer_delay < 2) {timer_delay = 2;} //Range check - 2 second minimum
              EEPROM.put(timercountEE, timer_delay); //Write out new value to non-volatile EEPROM after every button press
              shortBlink(); //Short blink of LED for each button press as confirmation
              }
            break;    
    }
        break;

    case RELEASED:
        //Some code here - No action on any button release
        break;

    case HOLD:
    switch (key) {
            case '1': //Attempt to start fill process
            if (fillmode == 0  && idlemode == 0 && blinkmode == 0) { //Start fill cycle if one not in progress, or wait delay or blink out modes not active
              fillmode = 1; //Set fillmode flag
              digitalWrite(ledPin, HIGH); //Turn on LED
              digitalWrite(valvePin, HIGH); //Turn on solenoid valve
              filltimer = coffeeTimer.setTimeout((timer_delay * 1000), callback_Fill); //Start fill timer
            }
            break;
            
            case '2': //Blink out current fill_timer setting if time increment button held down after adjustment
            if (fillmode == 0  && idlemode == 0 && blinkmode == 0) { //Start fill_timer blink out if one not in progress, or wait delay or blink out modes not active
              blink_filltime(); //Initiate LED blink of current 3-digit fill_timer setting
            }
            break;
            
            case '3': //Blink out current fill_timer setting if time increment button held down after adjustment
            if (fillmode == 0  && idlemode == 0 && blinkmode == 0) { //Start fill_timer blink out if one not in progress, or wait delay or blink out modes not active
              blink_filltime(); //Initiate LED blink of current 3-digit fill_timer setting
            }
            break;    
    }
    break;
  }
 }


void callback_Idle() //Callback function for idle timer fire
{
    idlemode = 0; //Clear idlemode flag to allow another fill cycle after delay for brew cycle compleion and reheating
    digitalWrite(ledPin, LOW); //Turn off LED, if left on by blinking
}

void callback_Fill() //Callback function for fill timer fire
{
    digitalWrite(ledPin, LOW); //Turn off LED
    digitalWrite(valvePin, LOW); //Turn off solenoid valve
    idlemode = 1; //Set idlemode flag
    fillmode = 0; //Clear fillmode flag after idlemode set, so no undefined state
    idletimer = coffeeTimer.setTimeout((idle_timer_interval * 1000), callback_Idle); //Start idle timer to insure delay before next brew attempt
}

void callback_Blink() //Callback function for repeating blink timer fire every 1/2 second
{
    if(idlemode == 1){ //Only perform subroutine if in idlemode
       if (digitalRead(ledPin) == HIGH) {digitalWrite(ledPin, LOW);} else{digitalWrite(ledPin, HIGH);} //Blink status LED if delaying before allowing next cycle.
    }
}

//Function to blink current value of fill_time on LED, in decimal
//Three digits always blinked, with delay between sequences
//A zero is indicated by 10 blinks - leading zeroes not suppressed

void blink_filltime() 
{
  blinkmode = 1; //Set blinkmode flag to lockout other functions
  //Serial.println(timer_delay);
  int hunpart = (timer_delay % 1000) / 100; //Calculate hundreds digit
  if (hunpart == 0) {hunpart = 10;} //Make 10 if zero for 10 blinks on zero value
  //Serial.println(hunpart);
  int decpart = ((timer_delay % 100) / 10); //Calculate tens digit
  if (decpart == 0) {decpart = 10;} //Make 10 if zero for 10 blinks on zero value
  //Serial.println(decpart);
  int onespart = (timer_delay % 10); //Calculate ones digit
  if (onespart == 0) {onespart = 10;} //Make 10 if zero for 10 blinks on zero value
  //Serial.println(onespart);
  //Serial.println();
  
  for (int i=0; i<hunpart; i++){  //Blink hundreds digit
    digitalWrite(ledPin, HIGH); //Turn on LED
    delay(50);
    digitalWrite(ledPin, LOW); //Turn off LED
    delay(350);
  }
  
  delay(1000); //Wait 1 second between hundreds and tens digit
  
  for (int i=0; i<decpart; i++){ //Blink tens digit
    digitalWrite(ledPin, HIGH); //Turn on LED
    delay(50);
    digitalWrite(ledPin, LOW); //Turn off LED
    delay(350);
  }
  
  delay(1000); //Wait 1 second between tens and ones digit
  
  for (int i=0; i<onespart; i++){ //Blink ones digit
    digitalWrite(ledPin, HIGH); //Turn on LED
    delay(50);
    digitalWrite(ledPin, LOW); //Turn off LED
    delay(350);
  }
  blinkmode = 0; //Clear blinkmode flag to allow other functions
}

void shortBlink() //Function to blink LED briefly on fill_timer adjustment short press
{
  delay(50);
  digitalWrite(ledPin, HIGH); //Turn on LED
  delay(50);
  digitalWrite(ledPin, LOW); //Turn off LED
  delay(50);
} 


